package remoteproxy;


import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;

/**
 * This application illustrates the use of sockets
 * to send a serialized object between two processes via a RemoteProxy
 * This app implements the client side.
 * @author hornick
 *
 */
public class ClientApp  {

	private Socket socket; // this client's socket
	private InetAddress netAddr;
	private ObjectInputStream fromServerStream;
	private ObjectOutputStream toServerStream;

	public static void main(String[] args) {
		ClientApp me = new ClientApp();
	}

	public ClientApp() {
		//	Create and connect a client-side socket to a pre-existing server socket 
		// localhost is 127.0.0.1
		byte[] addr = {(byte)155,(byte)92,(byte)177, (byte)81}; // 127.0.0.1 is the local loopback IP address

		// Create the socket connection
		try{
			netAddr = InetAddress.getByAddress(addr);
			System.out.println("Connecting to server...");
			socket = new Socket(netAddr, 4444); // connect to server at this IP address on the specified port
			System.out.println("Connected to server; starting to send and receive...");
			toServerStream = new ObjectOutputStream(socket.getOutputStream());
			fromServerStream = new ObjectInputStream(socket.getInputStream());
		} catch( UnknownHostException e ) {
			System.out.println("Unknown host");
			System.exit(1);
		} catch( IOException e ) {
			System.out.println("Can't connect to server-side socket.");
			System.exit(1);
		}

		
		// Create the remote proxy
		DateService ds = new DateProxy( fromServerStream ,toServerStream );

		while( true ) { // periodically ask the server for time and date
			// get time and date via the remote proxy from another process
			String today = ds.getDate();
			String time = ds.getTime();
			System.out.println("Today is " + today );
			System.out.println("Time in ms after 1/1/1970 is " + time );

			try {
				Thread.sleep(5000);
			} catch (InterruptedException e ) {
				// continue
			}
		}
	}	
}
