package remoteproxy;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.io.ObjectOutputStream;

/**
 * This application illustrates the use of sockets
 * to send a serialized object between two processes via a RemoteProxy.
 * This app implements the server side.
 * @author hornick
 *
 */
public class ServerApp {
	private ServerSocket serverSocket; // this server's socket
	private Socket clientSocket; // the socket on the client that attaches to this server's socket
	private ObjectOutputStream toClientStream;  // for writing to the client
	private ObjectInputStream fromClientStream; // for reading from the client

	public static void main(String[] args) {
		ServerApp me = new ServerApp();
	}

	public ServerApp() {
		//	Create server-side socket and wait for a single client to connect
		try {
			System.out.println("Creating server socket...");
			serverSocket = new ServerSocket(4444); // listen on this port
			//NOTE: This server can accept more than a single client connection, but we only accept one in this example
			System.out.println("Server socket created; waiting for client to connect...");
			clientSocket = serverSocket.accept();  // wait for a client to connect
			System.out.println("Client connected; starting to send and receive...");
			toClientStream = new ObjectOutputStream(clientSocket.getOutputStream());
			fromClientStream = new ObjectInputStream(clientSocket.getInputStream());
		} catch( UnknownHostException e ) {
			System.out.println("Unknown host");
			System.exit(1);
		} catch( IOException e ) {
			System.out.println("Can't create server-side socket and/or connect to client-side socket.");
			System.exit(1);
		}

		processMessages();
	}

	/**
	 * Listen and receive data on a socket stream
	 */
	public  void processMessages( ) {
		String returnValue = "?";
		// Create a RealDateSubject
		DateService ds = new RealDateSubject();

		while(true) { // service incoming messages forever
			try {
				Message request = (Message)fromClientStream.readObject();
				
				// process the request by detemining what it is...
				if( request.getRequestID() == Message.DATEREQUEST ) {
					returnValue = ds.getDate();
				} else {
					returnValue = ds.getTime();
				}
				
				// create a response message
				Message response = new Message( returnValue );
				
				// send it back
				toClientStream.writeObject(response);
				
			} catch( ClassNotFoundException e ) {
				System.out.println("server class cast error");
			} catch( IOException e ) {
				System.out.println("server read error" );
			}
		}
	}
}
